/*
 * Interprocess communication utilities.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/c/dataspace.h>
#include <l4/re/consts.h>
#include <l4/sys/task.h>

#include <mem/send_flexpage.h>

#include "ipc.h"



/* Make an L4 representation of the given flexpage. */

static l4_fpage_t ipc_get_fpage(SendFlexpage *send_flexpage)
{
    unsigned char flags = send_flexpage->flags;

    if (!flags)
        flags = L4RE_DS_MAP_FLAG_RO;

    return l4_fpage(send_flexpage->base_addr, send_flexpage->order, flags);
}

/* Make a representation of a flexpage for the IPC system. */

long ipc_prepare_flexpage(Flexpage *flexpage, offset_t offset,
                          offset_t max_offset, map_address_t hot_spot,
                          map_flags_t flags, l4_snd_fpage_t *region)
{
    SendFlexpage send_flexpage = flexpage->to_send(offset, hot_spot, flags,
                                                   max_offset);

    /* NOTE: Consider l4_fpage_invalid() as the fpage here. */

    if (!send_flexpage.order)
        return -L4_ERANGE;

    region->fpage = ipc_get_fpage(&send_flexpage);
    region->snd_base = hot_spot;

    return L4_EOK;
}

/* Unmap the given flexpage. */

void ipc_unmap_flexpage(Flexpage *flexpage)
{
    SendFlexpage send_flexpage = flexpage->to_unmap();

    l4_task_unmap(L4RE_THIS_TASK_CAP, ipc_get_fpage(&send_flexpage), L4_FP_OTHER_SPACES);
}

// vim: tabstop=4 expandtab shiftwidth=4
